@@ -36,7 +36,6 @@
 #include "rsvg-path-builder.h"
 #include "rsvg-filter.h"
 #include "rsvg-structure.h"
-#include "rsvg-image.h"
 
 #include <math.h>
 #include <string.h>
@@ -159,15 +158,24 @@ set_font_options_for_testing (PangoContext *context)
 static void
 create_font_config_for_testing (RsvgCairoRender *render)
 {
-    const char *font_path = SRCDIR "/tests/resources/LiberationSans-Regular.ttf";
+    const char *font_paths[] = {
+        SRCDIR "/tests/resources/Roboto-Regular.ttf",
+        SRCDIR "/tests/resources/Roboto-Italic.ttf",
+        SRCDIR "/tests/resources/Roboto-Bold.ttf",
+        SRCDIR "/tests/resources/Roboto-BoldItalic.ttf",
+    };
+
+    int i;
 
     if (render->font_config_for_testing != NULL)
         return;
 
     render->font_config_for_testing = FcConfigCreate ();
 
-    if (!FcConfigAppFontAddFile (render->font_config_for_testing, (const FcChar8 *) font_path)) {
-        g_error ("Could not load font file \"%s\" for tests; aborting", font_path);
+    for (i = 0; i < G_N_ELEMENTS(font_paths); i++) {
+        if (!FcConfigAppFontAddFile (render->font_config_for_testing, (const FcChar8 *) font_paths[i])) {
+            g_error ("Could not load font file \"%s\" for tests; aborting", font_paths[i]);
+        }
     }
 }
 
@@ -187,7 +195,7 @@ get_font_map_for_testing (RsvgCairoRender *render)
 #endif
 
 PangoContext *
-rsvg_cairo_create_pango_context (RsvgDrawingCtx * ctx)
+rsvg_cairo_get_pango_context (RsvgDrawingCtx * ctx)
 {
     PangoFontMap *fontmap;
     PangoContext *context;
@@ -519,7 +527,7 @@ rsvg_cairo_set_cairo_context (RsvgDrawingCtx *ctx, cairo_t *cr)
 }
 
 static void
-rsvg_cairo_generate_mask (cairo_t * cr, RsvgNode *node_mask, RsvgDrawingCtx *ctx, RsvgBbox *bbox)
+rsvg_cairo_generate_mask (cairo_t * cr, RsvgNode *mask, RsvgDrawingCtx *ctx, RsvgBbox *bbox)
 {
     RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
     cairo_surface_t *surface;
@@ -529,12 +537,13 @@ rsvg_cairo_generate_mask (cairo_t * cr, RsvgNode *node_mask, RsvgDrawingCtx *ctx
     guint32 width = render->width, height = render->height;
     guint32 rowstride = width * 4, row, i;
     cairo_matrix_t affinesave;
+    RsvgLength mask_x, mask_y, mask_w, mask_h;
     double sx, sy, sw, sh;
     gboolean nest = cr != render->initial_cr;
-    RsvgMask *self;
+    RsvgCoordUnits mask_units;
+    RsvgCoordUnits content_units;
 
-    g_assert (rsvg_node_get_type (node_mask) == RSVG_NODE_TYPE_MASK);
-    self = rsvg_rust_cnode_get_impl (node_mask);
+    g_assert (rsvg_node_get_type (mask) == RSVG_NODE_TYPE_MASK);
 
     surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
     if (cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS) {
@@ -545,22 +554,30 @@ rsvg_cairo_generate_mask (cairo_t * cr, RsvgNode *node_mask, RsvgDrawingCtx *ctx
     pixels = cairo_image_surface_get_data (surface);
     rowstride = cairo_image_surface_get_stride (surface);
 
-    if (self->maskunits == objectBoundingBox)
+    mask_units    = rsvg_node_mask_get_units (mask);
+    content_units = rsvg_node_mask_get_content_units (mask);
+
+    if (mask_units == objectBoundingBox)
         rsvg_drawing_ctx_push_view_box (ctx, 1, 1);
 
-    sx = rsvg_length_normalize (&self->x, ctx);
-    sy = rsvg_length_normalize (&self->y, ctx);
-    sw = rsvg_length_normalize (&self->width, ctx);
-    sh = rsvg_length_normalize (&self->height, ctx);
+    mask_x = rsvg_node_mask_get_x (mask);
+    mask_y = rsvg_node_mask_get_y (mask);
+    mask_w = rsvg_node_mask_get_width (mask);
+    mask_h = rsvg_node_mask_get_height (mask);
 
-    if (self->maskunits == objectBoundingBox)
+    sx = rsvg_length_normalize (&mask_x, ctx);
+    sy = rsvg_length_normalize (&mask_y, ctx);
+    sw = rsvg_length_normalize (&mask_w, ctx);
+    sh = rsvg_length_normalize (&mask_h, ctx);
+
+    if (mask_units == objectBoundingBox)
         rsvg_drawing_ctx_pop_view_box (ctx);
 
     mask_cr = cairo_create (surface);
     save_cr = render->cr;
     render->cr = mask_cr;
 
-    if (self->maskunits == objectBoundingBox)
+    if (mask_units == objectBoundingBox)
         rsvg_cairo_add_clipping_rect (ctx,
                                       sx * bbox->rect.width + bbox->rect.x,
                                       sy * bbox->rect.height + bbox->rect.y,
@@ -570,7 +587,7 @@ rsvg_cairo_generate_mask (cairo_t * cr, RsvgNode *node_mask, RsvgDrawingCtx *ctx
         rsvg_cairo_add_clipping_rect (ctx, sx, sy, sw, sh);
 
     /* Horribly dirty hack to have the bbox premultiplied to everything */
-    if (self->contentunits == objectBoundingBox) {
+    if (content_units == objectBoundingBox) {
         cairo_matrix_t bbtransform;
         RsvgState *mask_state;
 
@@ -582,7 +599,7 @@ rsvg_cairo_generate_mask (cairo_t * cr, RsvgNode *node_mask, RsvgDrawingCtx *ctx
                            bbox->rect.x,
                            bbox->rect.y);
 
-        mask_state = rsvg_node_get_state (node_mask);
+        mask_state = rsvg_node_get_state (mask);
 
         affinesave = mask_state->affine;
         cairo_matrix_multiply (&mask_state->affine, &bbtransform, &mask_state->affine);
@@ -590,15 +607,15 @@ rsvg_cairo_generate_mask (cairo_t * cr, RsvgNode *node_mask, RsvgDrawingCtx *ctx
     }
 
     rsvg_state_push (ctx);
-    rsvg_node_draw_children (node_mask, ctx, 0);
+    rsvg_node_draw_children (mask, ctx, 0);
     rsvg_state_pop (ctx);
 
-    if (self->contentunits == objectBoundingBox) {
+    if (content_units == objectBoundingBox) {
         RsvgState *mask_state;
 
         rsvg_drawing_ctx_pop_view_box (ctx);
 
-        mask_state = rsvg_node_get_state (node_mask);
+        mask_state = rsvg_node_get_state (mask);
         mask_state->affine = affinesave;
     }
 
@@ -656,9 +673,7 @@ rsvg_cairo_push_render_stack (RsvgDrawingCtx * ctx)
         RsvgNode *node;
         node = rsvg_drawing_ctx_acquire_node_of_type (ctx, rsvg_current_state (ctx)->clip_path, RSVG_NODE_TYPE_CLIP_PATH);
         if (node) {
-            RsvgClipPath *clip_path = rsvg_rust_cnode_get_impl (node);
-
-            switch (clip_path->units) {
+            switch (rsvg_node_clip_path_get_units (node)) {
             case userSpaceOnUse:
                 rsvg_cairo_clip (ctx, node, NULL);
                 break;
@@ -734,9 +749,7 @@ rsvg_cairo_pop_render_stack (RsvgDrawingCtx * ctx)
         RsvgNode *node;
         node = rsvg_drawing_ctx_acquire_node_of_type (ctx, rsvg_current_state (ctx)->clip_path, RSVG_NODE_TYPE_CLIP_PATH);
         if (node) {
-            RsvgClipPath *cp = rsvg_rust_cnode_get_impl (node);
-
-            if (cp->units == objectBoundingBox) {
+            if (rsvg_node_clip_path_get_units (node) == objectBoundingBox) {
                 lateclip = node;
             } else {
                 rsvg_drawing_ctx_release_node (ctx, node);
