@@ -53,7 +53,6 @@ typedef struct _RsvgState RsvgState;
 typedef struct _RsvgDefs RsvgDefs;
 typedef struct _RsvgNode RsvgNode;
 typedef struct _RsvgFilter RsvgFilter;
-typedef struct _RsvgNodeChars RsvgNodeChars;
 
 /* prepare for gettext */
 #ifndef _
@@ -120,12 +119,14 @@ struct RsvgSaxHandler {
     void (*free) (RsvgSaxHandler * self);
     void (*start_element) (RsvgSaxHandler * self, const char *name, RsvgPropertyBag * atts);
     void (*end_element) (RsvgSaxHandler * self, const char *name);
-    void (*characters) (RsvgSaxHandler * self, const char *ch, int len);
+    void (*characters) (RsvgSaxHandler * self, const char *ch, gssize len);
 };
 
 /* Reading state for an RsvgHandle */
 typedef enum {
     RSVG_HANDLE_STATE_START,
+    RSVG_HANDLE_STATE_EXPECTING_GZ_1,
+    RSVG_HANDLE_STATE_READING_COMPRESSED,
     RSVG_HANDLE_STATE_READING,
     RSVG_HANDLE_STATE_CLOSED_OK,
     RSVG_HANDLE_STATE_CLOSED_ERROR
@@ -181,7 +182,7 @@ struct RsvgHandlePrivate {
 
     gboolean in_loop;		/* see get_dimension() */
 
-    GInputStream *data_input_stream; /* for rsvg_handle_write of svgz data */
+    GInputStream *compressed_input_stream; /* for rsvg_handle_write of svgz data */
 
     gboolean is_testing; /* Are we being run from the test suite? */
 };
@@ -224,8 +225,8 @@ struct RsvgRender {
 
     void (*free) (RsvgRender * self);
 
-    PangoContext    *(*create_pango_context)    (RsvgDrawingCtx * ctx);
-    void             (*render_pango_layout)	    (RsvgDrawingCtx * ctx, PangoLayout *layout,
+    PangoContext    *(*get_pango_context)       (RsvgDrawingCtx * ctx);
+    void             (*render_pango_layout)	(RsvgDrawingCtx * ctx, PangoLayout *layout,
                                                  double x, double y);
     void             (*render_path_builder)     (RsvgDrawingCtx * ctx, RsvgPathBuilder *builder);
     void             (*render_surface)          (RsvgDrawingCtx * ctx, cairo_surface_t *surface,
@@ -282,7 +283,8 @@ typedef struct {
 } RsvgBbox;
 
 typedef enum {
-    objectBoundingBox, userSpaceOnUse
+    userSpaceOnUse,
+    objectBoundingBox
 } RsvgCoordUnits;
 
 /* Keep this in sync with rust/src/node.rs:NodeType */
@@ -422,9 +424,9 @@ void rsvg_node_foreach_child (RsvgNode *node, RsvgNodeForeachChildFn fn, gpointe
 G_GNUC_INTERNAL
 void rsvg_node_draw_children (RsvgNode *node, RsvgDrawingCtx *ctx, int dominate);
 
-struct _RsvgNodeChars {
-    GString *contents;
-};
+/* Implemented in rust/src/chars.rs */
+G_GNUC_INTERNAL
+void rsvg_node_chars_get_string (RsvgNode *node, const char **out_str, gsize *out_len);
 
 typedef void (*RsvgPropertyBagEnumFunc) (const char *key, const char *value, gpointer user_data);
 
@@ -448,8 +450,6 @@ GdkPixbuf *rsvg_pixbuf_from_data_with_size_data (const guchar * buff,
                                                  const char *base_uri, GError ** error);
 G_GNUC_INTERNAL
 gboolean     rsvg_eval_switch_attributes	(RsvgPropertyBag * atts, gboolean * p_has_cond);
-G_GNUC_INTERNAL
-gchar       *rsvg_get_base_uri_from_filename    (const gchar * file_name);
 
 G_GNUC_INTERNAL
 void rsvg_pop_discrete_layer    (RsvgDrawingCtx * ctx);
@@ -495,6 +495,9 @@ GdkPixbuf *rsvg_cairo_surface_to_pixbuf (cairo_surface_t *surface);
 G_GNUC_INTERNAL
 cairo_surface_t *rsvg_get_surface_of_node (RsvgDrawingCtx * ctx, RsvgNode * drawable, double w, double h);
 
+G_GNUC_INTERNAL
+cairo_surface_t *rsvg_cairo_surface_new_from_href (RsvgHandle *handle, const char *href, GError ** error);
+
 G_GNUC_INTERNAL
 void rsvg_drawing_ctx_free (RsvgDrawingCtx * handle);
 
@@ -555,6 +558,15 @@ G_GNUC_INTERNAL
 void rsvg_return_if_fail_warning (const char *pretty_function,
                                   const char *expression, GError ** error);
 
+G_GNUC_INTERNAL
+char *rsvg_handle_resolve_uri (RsvgHandle *handle,
+                               const char *uri);
+
+G_GNUC_INTERNAL
+gboolean rsvg_allow_load (GFile       *base_gfile,
+                          const char  *uri,
+                          GError     **error);
+
 G_GNUC_INTERNAL
 char *_rsvg_handle_acquire_data (RsvgHandle *handle,
                                  const char *uri,
