@@ -1,5 +1,5 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* vim: set sw=4 sts=4 ts=4 expandtab: */
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set sw=4 sts=4 expandtab: */
 /*
    rsvg-shapes.h: Draw SVG shapes
 
@@ -30,26 +30,37 @@
 #ifndef RSVG_SHAPES_H
 #define RSVG_SHAPES_H
 
-#include <cairo.h>
-
-#include "rsvg-structure.h"
+#include "rsvg-private.h"
 
 G_BEGIN_DECLS 
 
+/* Implemented in rust/src/shapes.rs */
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_path (const char *element_name);
+RsvgNode *rsvg_node_path_new (const char *element_name, RsvgNode *parent);
+
+/* Implemented in rust/src/shapes.rs */
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_polygon (const char *element_name);
+RsvgNode *rsvg_node_polygon_new (const char *element_name, RsvgNode *parent);
+
+/* Implemented in rust/src/shapes.rs */
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_polyline (const char *element_name);
+RsvgNode *rsvg_node_polyline_new (const char *element_name, RsvgNode *parent);
+
+/* Implemented in rust/src/shapes.rs */
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_line (const char *element_name);
+RsvgNode *rsvg_node_line_new (const char *element_name, RsvgNode *parent);
+
+/* Implemented in rust/src/shapes.rs */
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_rect (const char *element_name);
+RsvgNode *rsvg_node_rect_new (const char *element_name, RsvgNode *parent);
+
+/* Implemented in rust/src/shapes.rs */
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_circle (const char *element_name);
+RsvgNode *rsvg_node_circle_new (const char *element_name, RsvgNode *parent);
+
+/* Implemented in rust/src/shapes.rs */
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_ellipse (const char *element_name);
+RsvgNode *rsvg_node_ellipse_new (const char *element_name, RsvgNode *parent);
 
 G_END_DECLS
 
