@@ -700,15 +700,15 @@ rsvg_parse_style_pair (RsvgState * state,
         state->has_visible = TRUE;
         if (g_str_equal (value, "none"))
             state->visible = FALSE;
-        else if (!g_str_equal (value, "inherit") != 0)
+        else if (!g_str_equal (value, "inherit"))
             state->visible = TRUE;
         else
             state->has_visible = FALSE;
-	} else if (g_str_equal (name, "xml:space")) {
+    } else if (g_str_equal (name, "xml:space")) {
         state->has_space_preserve = TRUE;
         if (g_str_equal (value, "default"))
             state->space_preserve = FALSE;
-        else if (!g_str_equal (value, "preserve") == 0)
+        else if (g_str_equal (value, "preserve"))
             state->space_preserve = TRUE;
         else
             state->space_preserve = FALSE;
@@ -716,7 +716,7 @@ rsvg_parse_style_pair (RsvgState * state,
         state->has_visible = TRUE;
         if (g_str_equal (value, "visible"))
             state->visible = TRUE;
-        else if (!g_str_equal (value, "inherit") != 0)
+        else if (!g_str_equal (value, "inherit"))
             state->visible = FALSE;     /* collapse or hidden */
         else
             state->has_visible = FALSE;
@@ -1737,6 +1737,12 @@ rsvg_state_get_cond_true (RsvgState *state)
     return state->cond_true;
 }
 
+void
+rsvg_state_set_cond_true (RsvgState *state, gboolean cond_true)
+{
+    state->cond_true = cond_true;
+}
+
 RsvgCssColorSpec *
 rsvg_state_get_stop_color (RsvgState *state)
 {
