@@ -36,6 +36,15 @@
 /* what we use for text rendering depends on what cairo has to offer */
 #include <pango/pangocairo.h>
 
+/* Keep in sync with rust/src/space.rs:XmlSpace */
+typedef enum {
+    XML_SPACE_DEFAULT,
+    XML_SPACE_PRESERVE
+} XmlSpace;
+
+/* Implemented in rust/src/space.rs */
+extern char *rsvg_xml_space_normalize (XmlSpace mode, const char *s);
+
 typedef struct _RsvgNodeText RsvgNodeText;
 
 struct _RsvgNodeText {
@@ -51,79 +60,6 @@ struct _RsvgNodeTref {
     char *link;
 };
 
-char *
-rsvg_make_valid_utf8 (const char *str, int len)
-{
-    GString *string;
-    const char *remainder, *invalid;
-    int remaining_bytes, valid_bytes;
-
-    string = NULL;
-    remainder = str;
-
-    if (len < 0)
-        remaining_bytes = strlen (str);
-    else
-        remaining_bytes = len;
-
-    while (remaining_bytes != 0) {
-        if (g_utf8_validate (remainder, remaining_bytes, &invalid))
-            break;
-        valid_bytes = invalid - remainder;
-
-        if (string == NULL)
-            string = g_string_sized_new (remaining_bytes);
-
-        g_string_append_len (string, remainder, valid_bytes);
-        g_string_append_c (string, '?');
-
-        remaining_bytes -= valid_bytes + 1;
-        remainder = invalid + 1;
-    }
-
-    if (string == NULL)
-        return len < 0 ? g_strndup (str, len) : g_strdup (str);
-
-    g_string_append (string, remainder);
-
-    return g_string_free (string, FALSE);
-}
-
-static GString *
-_rsvg_text_chomp (RsvgState *state, GString * in, gboolean * lastwasspace)
-{
-    GString *out;
-    guint i;
-    out = g_string_new (in->str);
-
-    if (!state->space_preserve) {
-        for (i = 0; i < out->len;) {
-            if (out->str[i] == '\n')
-                g_string_erase (out, i, 1);
-            else
-                i++;
-        }
-
-        for (i = 0; i < out->len; i++)
-            if (out->str[i] == '\t')
-                out->str[i] = ' ';
-
-        for (i = 0; i < out->len;) {
-            if (out->str[i] == ' ' && *lastwasspace)
-                g_string_erase (out, i, 1);
-            else {
-                if (out->str[i] == ' ')
-                    *lastwasspace = TRUE;
-                else
-                    *lastwasspace = FALSE;
-                i++;
-            }
-        }
-    }
-
-    return out;
-}
-
 static void
 set_text_common_atts (RsvgNodeText *text, RsvgPropertyBag *atts)
 {
@@ -155,28 +91,46 @@ rsvg_node_text_set_atts (RsvgNode *node, gpointer impl, RsvgHandle *handle, Rsvg
 static void rsvg_text_render_text (RsvgDrawingCtx * ctx, const char *text, gdouble * x, gdouble * y);
 
 static void
-_rsvg_node_text_type_children (RsvgNode * self, RsvgDrawingCtx * ctx,
-                               gdouble * x, gdouble * y, gboolean * lastwasspace,
-                               gboolean usetextonly);
+draw_from_children (RsvgNode       *self,
+                    RsvgDrawingCtx *ctx,
+                    gdouble        *x,
+                    gdouble        *y,
+                    gboolean        usetextonly);
 
 static void
-_rsvg_node_text_type_tspan (RsvgNode *node, RsvgNodeText *self, RsvgDrawingCtx *ctx,
-                            gdouble *x, gdouble *y, gboolean *lastwasspace,
-                            gboolean usetextonly);
+draw_tspan (RsvgNode       *node,
+            RsvgNodeText   *self,
+            RsvgDrawingCtx *ctx,
+            gdouble        *x,
+            gdouble        *y,
+            gboolean        usetextonly);
 
 static void
-_rsvg_node_text_type_tref (RsvgNodeTref * self, RsvgDrawingCtx * ctx,
-                           gdouble * x, gdouble * y, gboolean * lastwasspace,
-                           gboolean usetextonly);
+draw_tref (RsvgNodeTref   *self,
+           RsvgDrawingCtx *ctx,
+           gdouble        *x,
+           gdouble        *y,
+           gboolean        usetextonly);
 
 typedef struct {
     RsvgDrawingCtx *ctx;
     gdouble *x;
     gdouble *y;
-    gboolean *lastwasspace;
     gboolean usetextonly;
 } DrawTextClosure;
 
+static XmlSpace
+xml_space_from_current_state (RsvgDrawingCtx *ctx)
+{
+    RsvgState *state = rsvg_current_state (ctx);
+
+    if (state->space_preserve) {
+        return XML_SPACE_PRESERVE;
+    } else {
+        return XML_SPACE_DEFAULT;
+    }
+}
+
 static gboolean
 draw_text_child (RsvgNode *node, gpointer data)
 {
@@ -186,38 +140,42 @@ draw_text_child (RsvgNode *node, gpointer data)
     closure = data;
 
     if (type == RSVG_NODE_TYPE_CHARS) {
-        RsvgNodeChars *chars = rsvg_rust_cnode_get_impl (node);
-        GString *str = _rsvg_text_chomp (rsvg_current_state (closure->ctx), chars->contents, closure->lastwasspace);
-        rsvg_text_render_text (closure->ctx, str->str, closure->x, closure->y);
-        g_string_free (str, TRUE);
+        const char *chars_str;
+        gsize chars_len;
+        GString *string;
+        char *chomped;
+
+        rsvg_node_chars_get_string (node, &chars_str, &chars_len);
+        string = g_string_new_len (chars_str, chars_len);
+
+        chomped = rsvg_xml_space_normalize (xml_space_from_current_state (closure->ctx), string->str);
+        g_string_free (string, TRUE);
+
+        rsvg_text_render_text (closure->ctx, chomped, closure->x, closure->y);
+        g_free (chomped);
     } else {
         if (closure->usetextonly) {
-            _rsvg_node_text_type_children (node,
-                                           closure->ctx,
-                                           closure->x,
-                                           closure->y,
-                                           closure->lastwasspace,
-                                           closure->usetextonly);
+            draw_from_children (node,
+                                closure->ctx,
+                                closure->x,
+                                closure->y,
+                                closure->usetextonly);
         } else {
             if (type == RSVG_NODE_TYPE_TSPAN) {
                 RsvgNodeText *tspan = rsvg_rust_cnode_get_impl (node);
-                rsvg_state_push (closure->ctx);
-                _rsvg_node_text_type_tspan (node,
-                                            tspan,
-                                            closure->ctx,
-                                            closure->x,
-                                            closure->y,
-                                            closure->lastwasspace,
-                                            closure->usetextonly);
-                rsvg_state_pop (closure->ctx);
+                draw_tspan (node,
+                            tspan,
+                            closure->ctx,
+                            closure->x,
+                            closure->y,
+                            closure->usetextonly);
             } else if (type == RSVG_NODE_TYPE_TREF) {
                 RsvgNodeTref *tref = rsvg_rust_cnode_get_impl (node);
-                _rsvg_node_text_type_tref (tref,
-                                           closure->ctx,
-                                           closure->x,
-                                           closure->y,
-                                           closure->lastwasspace,
-                                           closure->usetextonly);
+                draw_tref (tref,
+                           closure->ctx,
+                           closure->x,
+                           closure->y,
+                           closure->usetextonly);
             }
         }
     }
@@ -227,9 +185,11 @@ draw_text_child (RsvgNode *node, gpointer data)
 
 /* This function is responsible of selecting render for a text element including its children and giving it the drawing context */
 static void
-_rsvg_node_text_type_children (RsvgNode * self, RsvgDrawingCtx * ctx,
-                               gdouble * x, gdouble * y, gboolean * lastwasspace,
-                               gboolean usetextonly)
+draw_from_children (RsvgNode       *self,
+                    RsvgDrawingCtx *ctx,
+                    gdouble        *x,
+                    gdouble        *y,
+                    gboolean        usetextonly)
 {
     DrawTextClosure closure;
 
@@ -238,7 +198,6 @@ _rsvg_node_text_type_children (RsvgNode * self, RsvgDrawingCtx * ctx,
     closure.ctx = ctx;
     closure.x = x;
     closure.y = y;
-    closure.lastwasspace = lastwasspace;
     closure.usetextonly = usetextonly;
 
     rsvg_node_foreach_child (self, draw_text_child, &closure);
@@ -247,29 +206,29 @@ _rsvg_node_text_type_children (RsvgNode * self, RsvgDrawingCtx * ctx,
 }
 
 static gboolean
-_rsvg_node_text_length_children (RsvgNode * self, RsvgDrawingCtx * ctx,
-                                 gdouble * length, gboolean * lastwasspace,
-                                 gboolean usetextonly);
-
-static gboolean
-_rsvg_node_text_length_tref (RsvgNodeTref * self, RsvgDrawingCtx * ctx,
-                             gdouble * x, gboolean * lastwasspace,
-                             gboolean usetextonly);
-
-static gboolean
-_rsvg_node_text_length_tspan (RsvgNode *node,
-                              RsvgNodeText *self,
+compute_length_from_children (RsvgNode       *self,
                               RsvgDrawingCtx *ctx,
-                              gdouble *x,
-                              gboolean *lastwasspace,
-                              gboolean usetextonly);
+                              gdouble        *length,
+                              gboolean        usetextonly);
 
-static gdouble rsvg_text_length_text_as_string (RsvgDrawingCtx * ctx, const char *text);
+static gboolean
+length_from_tref (RsvgNodeTref   *self,
+                  RsvgDrawingCtx *ctx,
+                  gdouble        *x,
+                  gboolean        usetextonly);
+
+static gboolean
+length_from_tspan (RsvgNode       *node,
+                   RsvgNodeText   *self,
+                   RsvgDrawingCtx *ctx,
+                   gdouble        *x,
+                   gboolean        usetextonly);
+
+static gdouble measure_text (RsvgDrawingCtx * ctx, const char *text);
 
 typedef struct {
     RsvgDrawingCtx *ctx;
     gdouble *length;
-    gboolean *lastwasspace;
     gboolean usetextonly;
     gboolean done;
 } ChildrenLengthClosure;
@@ -288,33 +247,39 @@ compute_child_length (RsvgNode *node, gpointer data)
     rsvg_state_reinherit_top (closure->ctx, rsvg_node_get_state (node), 0);
 
     if (type == RSVG_NODE_TYPE_CHARS) {
-        RsvgNodeChars *chars = rsvg_rust_cnode_get_impl (node);
-        GString *str = _rsvg_text_chomp (rsvg_current_state (closure->ctx), chars->contents, closure->lastwasspace);
-        *closure->length += rsvg_text_length_text_as_string (closure->ctx, str->str);
-        g_string_free (str, TRUE);
+        const char *chars_str;
+        gsize chars_len;
+        GString *string;
+        char *chomped;
+
+        rsvg_node_chars_get_string (node, &chars_str, &chars_len);
+        string = g_string_new_len (chars_str, chars_len);
+
+        chomped = rsvg_xml_space_normalize (xml_space_from_current_state (closure->ctx), string->str);
+        g_string_free (string, TRUE);
+
+        *closure->length += measure_text (closure->ctx, chomped);
+        g_free (chomped);
     } else {
         if (closure->usetextonly) {
-            done = _rsvg_node_text_length_children (node,
-                                                    closure->ctx,
-                                                    closure->length,
-                                                    closure->lastwasspace,
-                                                    closure->usetextonly);
+            done = compute_length_from_children (node,
+                                                 closure->ctx,
+                                                 closure->length,
+                                                 closure->usetextonly);
         } else {
             if (type == RSVG_NODE_TYPE_TSPAN) {
                 RsvgNodeText *tspan = rsvg_rust_cnode_get_impl (node);
-                done = _rsvg_node_text_length_tspan (node,
-                                                     tspan,
-                                                     closure->ctx,
-                                                     closure->length,
-                                                     closure->lastwasspace,
-                                                     closure->usetextonly);
+                done = length_from_tspan (node,
+                                          tspan,
+                                          closure->ctx,
+                                          closure->length,
+                                          closure->usetextonly);
             } else if (type == RSVG_NODE_TYPE_TREF) {
                 RsvgNodeTref *tref = rsvg_rust_cnode_get_impl (node);
-                done = _rsvg_node_text_length_tref (tref,
-                                                    closure->ctx,
-                                                    closure->length,
-                                                    closure->lastwasspace,
-                                                    closure->usetextonly);
+                done = length_from_tref (tref,
+                                         closure->ctx,
+                                         closure->length,
+                                         closure->usetextonly);
             }
         }
     }
@@ -326,15 +291,15 @@ compute_child_length (RsvgNode *node, gpointer data)
 }
 
 static gboolean
-_rsvg_node_text_length_children (RsvgNode * self, RsvgDrawingCtx * ctx,
-                                 gdouble * length, gboolean * lastwasspace,
-                                 gboolean usetextonly)
+compute_length_from_children (RsvgNode       *self,
+                              RsvgDrawingCtx *ctx,
+                              gdouble        *length,
+                              gboolean        usetextonly)
 {
     ChildrenLengthClosure closure;
 
     closure.ctx = ctx;
     closure.length = length;
-    closure.lastwasspace = lastwasspace;
     closure.usetextonly = usetextonly;
     closure.done = FALSE;
 
@@ -349,7 +314,6 @@ rsvg_node_text_draw (RsvgNode *node, gpointer impl, RsvgDrawingCtx *ctx, int dom
 {
     RsvgNodeText *text = impl;
     double x, y, dx, dy, length = 0;
-    gboolean lastwasspace = TRUE;
 
     rsvg_state_reinherit_top (ctx, rsvg_node_get_state (node), dominate);
 
@@ -359,7 +323,7 @@ rsvg_node_text_draw (RsvgNode *node, gpointer impl, RsvgDrawingCtx *ctx, int dom
     dy = rsvg_length_normalize (&text->dy, ctx);
 
     if (rsvg_current_state (ctx)->text_anchor != TEXT_ANCHOR_START) {
-        _rsvg_node_text_length_children (node, ctx, &length, &lastwasspace, FALSE);
+        compute_length_from_children (node, ctx, &length, FALSE);
         if (rsvg_current_state (ctx)->text_anchor == TEXT_ANCHOR_MIDDLE)
             length /= 2;
     }
@@ -379,8 +343,7 @@ rsvg_node_text_draw (RsvgNode *node, gpointer impl, RsvgDrawingCtx *ctx, int dom
     x += dx;
     y += dy;
 
-    lastwasspace = TRUE;
-    _rsvg_node_text_type_children (node, ctx, &x, &y, &lastwasspace, FALSE);
+    draw_from_children (node, ctx, &x, &y, FALSE);
 }
 
 RsvgNode *
@@ -401,20 +364,24 @@ rsvg_new_text (const char *element_name, RsvgNode *parent)
 }
 
 static void
-_rsvg_node_text_type_tspan (RsvgNode *node, RsvgNodeText *self, RsvgDrawingCtx *ctx,
-                            gdouble *x, gdouble *y, gboolean *lastwasspace,
-                            gboolean usetextonly)
+draw_tspan (RsvgNode       *node,
+            RsvgNodeText   *self,
+            RsvgDrawingCtx *ctx,
+            gdouble        *x,
+            gdouble        *y,
+            gboolean        usetextonly)
 {
     double dx, dy, length = 0;
 
+    rsvg_state_push (ctx);
+
     rsvg_state_reinherit_top (ctx, rsvg_node_get_state (node), 0);
 
     dx = rsvg_length_normalize (&self->dx, ctx);
     dy = rsvg_length_normalize (&self->dy, ctx);
 
     if (rsvg_current_state (ctx)->text_anchor != TEXT_ANCHOR_START) {
-        gboolean lws = *lastwasspace;
-        _rsvg_node_text_length_children (node, ctx, &length, &lws, usetextonly);
+        compute_length_from_children (node, ctx, &length, usetextonly);
         if (rsvg_current_state (ctx)->text_anchor == TEXT_ANCHOR_MIDDLE)
             length /= 2;
     }
@@ -442,16 +409,17 @@ _rsvg_node_text_type_tspan (RsvgNode *node, RsvgNodeText *self, RsvgDrawingCtx *
         }
     }
     *y += dy;
-    _rsvg_node_text_type_children (node, ctx, x, y, lastwasspace, usetextonly);
+    draw_from_children (node, ctx, x, y, usetextonly);
+
+    rsvg_state_pop (ctx);
 }
 
 static gboolean
-_rsvg_node_text_length_tspan (RsvgNode *node,
-                              RsvgNodeText *self,
-                              RsvgDrawingCtx *ctx,
-                              gdouble *length,
-                              gboolean *lastwasspace,
-                              gboolean usetextonly)
+length_from_tspan (RsvgNode       *node,
+                   RsvgNodeText   *self,
+                   RsvgDrawingCtx *ctx,
+                   gdouble        *length,
+                   gboolean        usetextonly)
 {
     if (self->x_specified || self->y_specified)
         return TRUE;
@@ -461,8 +429,7 @@ _rsvg_node_text_length_tspan (RsvgNode *node,
     else
         *length += rsvg_length_normalize (&self->dx, ctx);
 
-    return _rsvg_node_text_length_children (node, ctx, length,
-                                            lastwasspace, usetextonly);
+    return compute_length_from_children (node, ctx, length, usetextonly);
 }
 
 static void
@@ -497,9 +464,11 @@ rsvg_new_tspan (const char *element_name, RsvgNode *parent)
 }
 
 static void
-_rsvg_node_text_type_tref (RsvgNodeTref * self, RsvgDrawingCtx * ctx,
-                           gdouble * x, gdouble * y, gboolean * lastwasspace,
-                           gboolean usetextonly)
+draw_tref (RsvgNodeTref   *self,
+           RsvgDrawingCtx *ctx,
+           gdouble        *x,
+           gdouble        *y,
+           gboolean        usetextonly)
 {
     RsvgNode *link;
 
@@ -509,14 +478,16 @@ _rsvg_node_text_type_tref (RsvgNodeTref * self, RsvgDrawingCtx * ctx,
     if (link == NULL)
       return;
 
-    _rsvg_node_text_type_children (link, ctx, x, y, lastwasspace, TRUE);
+    draw_from_children (link, ctx, x, y, TRUE);
 
     rsvg_drawing_ctx_release_node (ctx, link);
 }
 
 static gboolean
-_rsvg_node_text_length_tref (RsvgNodeTref * self, RsvgDrawingCtx * ctx, gdouble * x,
-                             gboolean * lastwasspace, gboolean usetextonly)
+length_from_tref (RsvgNodeTref   *self,
+                  RsvgDrawingCtx *ctx,
+                  gdouble        *x,
+                  gboolean        usetextonly)
 {
     gboolean result;
     RsvgNode *link;
@@ -527,7 +498,7 @@ _rsvg_node_text_length_tref (RsvgNodeTref * self, RsvgDrawingCtx * ctx, gdouble
     if (link == NULL)
       return FALSE;
 
-    result = _rsvg_node_text_length_children (link, ctx, x, lastwasspace, TRUE);
+    result = compute_length_from_children (link, ctx, x, TRUE);
 
     rsvg_drawing_ctx_release_node (ctx, link);
 
@@ -578,22 +549,6 @@ rsvg_new_tref (const char *element_name, RsvgNode *parent)
                                 rsvg_node_tref_free);
 }
 
-typedef struct _RsvgTextLayout RsvgTextLayout;
-
-struct _RsvgTextLayout {
-    PangoLayout *layout;
-    RsvgDrawingCtx *ctx;
-    TextAnchor anchor;
-    gdouble x, y;
-};
-
-static void
-rsvg_text_layout_free (RsvgTextLayout * layout)
-{
-    g_object_unref (layout->layout);
-    g_free (layout);
-}
-
 static PangoLayout *
 rsvg_text_create_layout (RsvgDrawingCtx * ctx, const char *text, PangoContext * context)
 {
@@ -668,29 +623,7 @@ rsvg_text_create_layout (RsvgDrawingCtx * ctx, const char *text, PangoContext *
     return layout;
 }
 
-
-static RsvgTextLayout *
-rsvg_text_layout_new (RsvgDrawingCtx * ctx, const char *text)
-{
-    RsvgState *state;
-    RsvgTextLayout *layout;
-
-    state = rsvg_current_state (ctx);
-
-    if (ctx->pango_context == NULL)
-        ctx->pango_context = ctx->render->create_pango_context (ctx);
-
-    layout = g_new0 (RsvgTextLayout, 1);
-
-    layout->layout = rsvg_text_create_layout (ctx, text, ctx->pango_context);
-    layout->ctx = ctx;
-
-    layout->anchor = state->text_anchor;
-
-    return layout;
-}
-
-void
+static void
 rsvg_text_render_text (RsvgDrawingCtx * ctx, const char *text, gdouble * x, gdouble * y)
 {
     PangoContext *context;
@@ -706,7 +639,7 @@ rsvg_text_render_text (RsvgDrawingCtx * ctx, const char *text, gdouble * x, gdou
     if (state->font_size.length == 0)
         return;
 
-    context = ctx->render->create_pango_context (ctx);
+    context = ctx->render->get_pango_context (ctx);
     layout = rsvg_text_create_layout (ctx, text, context);
     pango_layout_get_size (layout, &w, &h);
     iter = pango_layout_get_iter (layout);
@@ -731,26 +664,22 @@ rsvg_text_render_text (RsvgDrawingCtx * ctx, const char *text, gdouble * x, gdou
 }
 
 static gdouble
-rsvg_text_layout_width (RsvgTextLayout * layout)
+measure_text (RsvgDrawingCtx * ctx, const char *text)
 {
+    PangoContext *context;
+    PangoLayout *layout;
     gint width;
+    gdouble scaled_width;
 
-    pango_layout_get_size (layout->layout, &width, NULL);
+    context = ctx->render->get_pango_context (ctx);
 
-    return width / (double)PANGO_SCALE;
-}
+    layout = rsvg_text_create_layout (ctx, text, context);
 
-static gdouble
-rsvg_text_length_text_as_string (RsvgDrawingCtx * ctx, const char *text)
-{
-    RsvgTextLayout *layout;
-    gdouble x;
+    pango_layout_get_size (layout, &width, NULL);
+    scaled_width = width / (double)PANGO_SCALE;
 
-    layout = rsvg_text_layout_new (ctx, text);
-    layout->x = layout->y = 0;
+    g_object_unref (layout);
+    g_object_unref (context);
 
-    x = rsvg_text_layout_width (layout);
-
-    rsvg_text_layout_free (layout);
-    return x;
+    return scaled_width;
 }
