@@ -64,6 +64,7 @@ extern "C" {
     fn rsvg_state_is_overflow (state: *const RsvgState) -> glib_sys::gboolean;
     fn rsvg_state_has_overflow (state: *const RsvgState) -> glib_sys::gboolean;
     fn rsvg_state_get_cond_true (state: *const RsvgState) -> glib_sys::gboolean;
+    fn rsvg_state_set_cond_true (state: *const RsvgState, cond_true: glib_sys::gboolean);
     fn rsvg_state_get_stop_color (state: *const RsvgState) -> *const ColorSpec;
     fn rsvg_state_get_stop_opacity (state: *const RsvgState) -> *const OpacitySpec;
     fn rsvg_state_get_current_color (state: *const RsvgState) -> u32;
@@ -80,6 +81,12 @@ extern "C" {
 
     fn rsvg_render_path_builder (draw_ctx: *const RsvgDrawingCtx,
                                  builder: *const RsvgPathBuilder);
+    fn rsvg_render_surface (draw_ctx: *const RsvgDrawingCtx,
+                            surface: *const cairo_sys::cairo_surface_t,
+                            x: f64,
+                            y: f64,
+                            w: f64,
+                            h: f64);
 
     fn rsvg_cairo_get_cairo_context (draw_ctx: *const RsvgDrawingCtx) -> *mut cairo_sys::cairo_t;
     fn rsvg_cairo_set_cairo_context (draw_ctx: *const RsvgDrawingCtx, cr: *const cairo_sys::cairo_t);
@@ -154,6 +161,15 @@ pub fn render_path_builder (draw_ctx: *const RsvgDrawingCtx,
     unsafe { rsvg_render_path_builder (draw_ctx, builder); }
 }
 
+pub fn render_surface (draw_ctx: *const RsvgDrawingCtx,
+                       surface: &cairo::ImageSurface,
+                       x: f64,
+                       y: f64,
+                       w: f64,
+                       h: f64) {
+    unsafe { rsvg_render_surface (draw_ctx, surface.to_raw_none(), x, y, w, h); }
+}
+
 pub fn get_cairo_context (draw_ctx: *const RsvgDrawingCtx) -> cairo::Context {
     unsafe {
         let raw_cr = rsvg_cairo_get_cairo_context (draw_ctx);
@@ -230,6 +246,10 @@ pub fn state_get_cond_true (state: *const RsvgState) -> bool {
     unsafe { from_glib (rsvg_state_get_cond_true (state)) }
 }
 
+pub fn state_set_cond_true (state: *const RsvgState, cond_true: bool) {
+    unsafe { rsvg_state_set_cond_true (state, cond_true.to_glib()); }
+}
+
 pub fn state_push (draw_ctx: *const RsvgDrawingCtx) {
     unsafe {
         rsvg_state_push (draw_ctx);
