@@ -8,12 +8,14 @@ use std::cell::RefCell;
 use cairo::MatrixTrait;
 
 use bbox::*;
+use coord_units::CoordUnits;
 use drawing_ctx;
 use drawing_ctx::RsvgDrawingCtx;
 use handle::RsvgHandle;
 use length::*;
 use node::*;
 use paint_server::*;
+use parsers::Parse;
 use property_bag;
 use property_bag::*;
 use stop::*;
@@ -21,18 +23,20 @@ use util::*;
 
 
 #[derive(Copy, Clone)]
-pub struct ColorStop {
+struct ColorStop {
     pub offset: f64,
     pub rgba:   u32
 }
 
+coord_units!(GradientUnits, CoordUnits::ObjectBoundingBox);
+
 /* Any of the attributes in gradient elements may be omitted.  In turn, the missing
  * ones can be inherited from the gradient referenced by its "fallback" IRI.  We
  * represent these possibly-missing attributes as Option<foo>.
  */
 #[derive(Clone)]
-pub struct GradientCommon {
-    pub units:    Option<PaintServerUnits>,
+struct GradientCommon {
+    pub units:    Option<GradientUnits>,
     pub affine:   Option<cairo::Matrix>,
     pub spread:   Option<PaintServerSpread>,
     pub fallback: Option<String>,
@@ -40,7 +44,7 @@ pub struct GradientCommon {
 }
 
 #[derive(Copy, Clone)]
-pub enum GradientVariant {
+enum GradientVariant {
     Linear {
         x1: Option<RsvgLength>,
         y1: Option<RsvgLength>,
@@ -58,7 +62,7 @@ pub enum GradientVariant {
 }
 
 #[derive(Clone)]
-pub struct Gradient {
+struct Gradient {
     pub common: GradientCommon,
     pub variant: GradientVariant
 }
@@ -117,7 +121,7 @@ impl GradientCommon {
     fn resolve_from_defaults (&mut self) {
         /* These are per the spec */
 
-        fallback_to! (self.units,  Some (PaintServerUnits::default ()));
+        fallback_to! (self.units,  Some (GradientUnits::default ()));
         fallback_to! (self.affine, Some (cairo::Matrix::identity ()));
         fallback_to! (self.spread, Some (PaintServerSpread::default ()));
         fallback_to! (self.stops,  Some (Vec::<ColorStop>::new ())); // empty array of color stops
@@ -361,7 +365,7 @@ fn set_common_on_pattern<P: cairo::Pattern + cairo::Gradient> (gradient: &Gradie
 
     let units = gradient.common.units.unwrap ();
 
-    if units == PaintServerUnits::ObjectBoundingBox {
+    if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
         let bbox_matrix = cairo::Matrix::new (bbox.rect.width, 0.0,
                                               0.0, bbox.rect.height,
                                               bbox.rect.x, bbox.rect.y);
@@ -384,7 +388,7 @@ fn set_linear_gradient_on_pattern (gradient: &Gradient,
     if let GradientVariant::Linear { x1, y1, x2, y2 } = gradient.variant {
         let units = gradient.common.units.unwrap ();
 
-        if units == PaintServerUnits::ObjectBoundingBox {
+        if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
             drawing_ctx::push_view_box (draw_ctx, 1.0, 1.0);
         }
 
@@ -393,7 +397,7 @@ fn set_linear_gradient_on_pattern (gradient: &Gradient,
                                                       x2.as_ref ().unwrap ().normalize (draw_ctx),
                                                       y2.as_ref ().unwrap ().normalize (draw_ctx));
 
-        if units == PaintServerUnits::ObjectBoundingBox {
+        if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
             drawing_ctx::pop_view_box (draw_ctx);
         }
 
@@ -466,7 +470,7 @@ fn set_radial_gradient_on_pattern (gradient: &Gradient,
     if let GradientVariant::Radial { cx, cy, r, fx, fy } = gradient.variant {
         let units = gradient.common.units.unwrap ();
 
-        if units == PaintServerUnits::ObjectBoundingBox {
+        if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
             drawing_ctx::push_view_box (draw_ctx, 1.0, 1.0);
         }
 
@@ -480,7 +484,7 @@ fn set_radial_gradient_on_pattern (gradient: &Gradient,
 
         let mut pattern = cairo::RadialGradient::new (new_fx, new_fy, 0.0, n_cx, n_cy, n_r);
 
-        if units == PaintServerUnits::ObjectBoundingBox {
+        if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
             drawing_ctx::pop_view_box (draw_ctx);
         }
 
@@ -556,9 +560,9 @@ impl NodeTrait for NodeGradient {
 
         // Attributes common to linear and radial gradients
 
-        g.common.units    = property_bag::parse_or_none (pbag, "gradientUnits")?;
-        g.common.affine   = property_bag::transform_or_none (pbag, "gradientTransform")?;
-        g.common.spread   = property_bag::parse_or_none (pbag, "spreadMethod")?;
+        g.common.units    = property_bag::parse_or_none (pbag, "gradientUnits", (), None)?;
+        g.common.affine   = property_bag::parse_or_none (pbag, "gradientTransform", (), None)?;
+        g.common.spread   = property_bag::parse_or_none (pbag, "spreadMethod", (), None)?;
         g.common.fallback = property_bag::lookup (pbag, "xlink:href");
 
         // Attributes specific to each gradient type.  The defaults mandated by the spec
@@ -567,20 +571,20 @@ impl NodeTrait for NodeGradient {
         match node.get_type () {
             NodeType::LinearGradient => {
                 g.variant = GradientVariant::Linear {
-                    x1: property_bag::length_or_none (pbag, "x1", LengthDir::Horizontal)?,
-                    y1: property_bag::length_or_none (pbag, "y1", LengthDir::Vertical)?,
-                    x2: property_bag::length_or_none (pbag, "x2", LengthDir::Horizontal)?,
-                    y2: property_bag::length_or_none (pbag, "y2", LengthDir::Vertical)?
+                    x1: property_bag::parse_or_none (pbag, "x1", LengthDir::Horizontal, None)?,
+                    y1: property_bag::parse_or_none (pbag, "y1", LengthDir::Vertical, None)?,
+                    x2: property_bag::parse_or_none (pbag, "x2", LengthDir::Horizontal, None)?,
+                    y2: property_bag::parse_or_none (pbag, "y2", LengthDir::Vertical, None)?
                 };
             },
 
             NodeType::RadialGradient => {
                 g.variant = GradientVariant::Radial {
-                    cx: property_bag::length_or_none (pbag, "cx", LengthDir::Horizontal)?,
-                    cy: property_bag::length_or_none (pbag, "cy", LengthDir::Vertical)?,
-                    r:  property_bag::length_or_none (pbag, "r",  LengthDir::Both)?,
-                    fx: property_bag::length_or_none (pbag, "fx", LengthDir::Horizontal)?,
-                    fy: property_bag::length_or_none (pbag, "fy", LengthDir::Vertical)?
+                    cx: property_bag::parse_or_none (pbag, "cx", LengthDir::Horizontal, None)?,
+                    cy: property_bag::parse_or_none (pbag, "cy", LengthDir::Vertical, None)?,
+                    r:  property_bag::parse_or_none (pbag, "r",  LengthDir::Both, None)?,
+                    fx: property_bag::parse_or_none (pbag, "fx", LengthDir::Horizontal, None)?,
+                    fy: property_bag::parse_or_none (pbag, "fy", LengthDir::Vertical, None)?
                 };
             },
 
