@@ -1,10 +1,8 @@
-use ::cairo;
 use ::glib::translate::*;
 use ::libc;
 
 use std::cell::Cell;
 use std::f64::consts::*;
-use std::str::FromStr;
 
 use cairo::MatrixTrait;
 
@@ -17,6 +15,7 @@ use length::*;
 use node::*;
 use path_builder::*;
 use parsers;
+use parsers::Parse;
 use parsers::ParseError;
 use property_bag;
 use property_bag::*;
@@ -37,10 +36,11 @@ impl Default for MarkerUnits {
     }
 }
 
-impl FromStr for MarkerUnits {
+impl Parse for MarkerUnits {
+    type Data = ();
     type Err = AttributeError;
 
-    fn from_str (s: &str) -> Result <MarkerUnits, AttributeError> {
+    fn parse (s: &str, _: ()) -> Result <MarkerUnits, AttributeError> {
         match s {
             "userSpaceOnUse" => Ok (MarkerUnits::UserSpaceOnUse),
             "strokeWidth"    => Ok (MarkerUnits::StrokeWidth),
@@ -63,10 +63,11 @@ impl Default for MarkerOrient {
     }
 }
 
-impl FromStr for MarkerOrient {
+impl Parse for MarkerOrient {
+    type Data = ();
     type Err = AttributeError;
 
-    fn from_str (s: &str) -> Result <MarkerOrient, AttributeError> {
+    fn parse (s: &str, _: ()) -> Result <MarkerOrient, AttributeError> {
         match s {
             "auto" => Ok (MarkerOrient::Auto),
             _      => parsers::angle_degrees (s)
@@ -95,17 +96,17 @@ impl NodeMarker {
             units:  Cell::new (MarkerUnits::default ()),
             ref_x:  Cell::new (RsvgLength::default ()),
             ref_y:  Cell::new (RsvgLength::default ()),
-            width:  Cell::new (NodeMarker::get_default_size ()),
-            height: Cell::new (NodeMarker::get_default_size ()),
+            width:  Cell::new (NodeMarker::get_default_size (LengthDir::Horizontal)),
+            height: Cell::new (NodeMarker::get_default_size (LengthDir::Vertical)),
             orient: Cell::new (MarkerOrient::default ()),
             aspect: Cell::new (AspectRatio::default ()),
             vbox:   Cell::new (None)
         }
     }
 
-    fn get_default_size () -> RsvgLength {
+    fn get_default_size (dir: LengthDir) -> RsvgLength {
         // per the spec
-        RsvgLength::parse ("3", LengthDir::Both).unwrap ()
+        RsvgLength::parse ("3", dir).unwrap()
     }
 
     fn render (&self,
@@ -119,17 +120,13 @@ impl NodeMarker {
         let marker_width = self.width.get ().normalize (draw_ctx);
         let marker_height = self.height.get ().normalize (draw_ctx);
 
-        let mut affine = cairo::Matrix::identity ();
+        let mut affine = drawing_ctx::get_current_state_affine (draw_ctx);
         affine.translate (xpos, ypos);
 
-        affine = cairo::Matrix::multiply (&affine, &drawing_ctx::get_current_state_affine (draw_ctx));
-
-        let rotation: f64;
-
-        match self.orient.get () {
-            MarkerOrient::Auto =>        { rotation = computed_angle; },
-            MarkerOrient::Degrees (d) => { rotation = d * PI / 180.0; }
-        }
+        let rotation = match self.orient.get () {
+            MarkerOrient::Auto =>        computed_angle,
+            MarkerOrient::Degrees (d) => d * PI / 180.0,
+        };
 
         affine.rotate (rotation);
 
@@ -145,8 +142,10 @@ impl NodeMarker {
             affine.scale (w / vbox.0.width, h / vbox.0.height);
 
             drawing_ctx::push_view_box (draw_ctx, vbox.0.width, vbox.0.height);
+            drawing_ctx::push_discrete_layer (draw_ctx);
         } else {
             drawing_ctx::push_view_box (draw_ctx, marker_width, marker_height);
+            drawing_ctx::push_discrete_layer (draw_ctx);
         }
 
         affine.translate (-self.ref_x.get ().normalize (draw_ctx),
@@ -160,7 +159,6 @@ impl NodeMarker {
 
         drawing_ctx::set_current_state_affine (draw_ctx, affine);
 
-        drawing_ctx::push_discrete_layer (draw_ctx);
 
         let state = drawing_ctx::get_current_state (draw_ctx);
 
@@ -190,20 +188,25 @@ impl NodeMarker {
 
 impl NodeTrait for NodeMarker {
     fn set_atts (&self, _: &RsvgNode, _: *const RsvgHandle, pbag: *const RsvgPropertyBag) -> NodeResult {
-        self.units.set (property_bag::parse_or_default (pbag, "markerUnits")?);
+        self.units.set (property_bag::parse_or_default (pbag, "markerUnits", (), None)?);
 
-        self.ref_x.set (property_bag::length_or_default (pbag, "refX", LengthDir::Horizontal)?);
-        self.ref_y.set (property_bag::length_or_default (pbag, "refY", LengthDir::Vertical)?);
+        self.ref_x.set (property_bag::parse_or_default (pbag, "refX", LengthDir::Horizontal, None)?);
+        self.ref_y.set (property_bag::parse_or_default (pbag, "refY", LengthDir::Vertical, None)?);
 
-        self.width.set (property_bag::lookup (pbag, "markerWidth").map_or (NodeMarker::get_default_size (),
-                                                                           |v| RsvgLength::parse (&v, LengthDir::Horizontal).unwrap_or (NodeMarker::get_default_size ())));
-        self.height.set (property_bag::lookup (pbag, "markerHeight").map_or (NodeMarker::get_default_size (),
-                                                                             |v| RsvgLength::parse (&v, LengthDir::Vertical).unwrap_or (NodeMarker::get_default_size ())));
+        self.width.set (property_bag::parse_or_value (pbag, "markerWidth",
+                                                      LengthDir::Horizontal,
+                                                      NodeMarker::get_default_size (LengthDir::Horizontal),
+                                                      Some(RsvgLength::check_nonnegative))?);
 
-        self.orient.set (property_bag::parse_or_default (pbag, "orient")?);
-        self.aspect.set (property_bag::parse_or_default (pbag, "preserveAspectRatio")?);
-        self.vbox.set   (property_bag::parse_or_none (pbag, "viewBox")?);
-        self.aspect.set (property_bag::parse_or_default (pbag, "preserveAspectRatio")?);
+        self.height.set (property_bag::parse_or_value (pbag, "markerHeight",
+                                                       LengthDir::Vertical,
+                                                       NodeMarker::get_default_size (LengthDir::Vertical),
+                                                       Some(RsvgLength::check_nonnegative))?);
+
+        self.orient.set (property_bag::parse_or_default (pbag, "orient", (), None)?);
+        self.aspect.set (property_bag::parse_or_default (pbag, "preserveAspectRatio", (), None)?);
+        self.vbox.set   (property_bag::parse_or_none (pbag, "viewBox", (), None)?);
+        self.aspect.set (property_bag::parse_or_default (pbag, "preserveAspectRatio", (), None)?);
 
         Ok (())
     }
@@ -762,32 +765,32 @@ mod parser_tests {
 
     #[test]
     fn parsing_invalid_marker_units_yields_error () {
-        assert! (is_parse_error (&MarkerUnits::from_str ("").map_err (|e| AttributeError::from (e))));
-        assert! (is_parse_error (&MarkerUnits::from_str ("foo").map_err (|e| AttributeError::from (e))));
+        assert! (is_parse_error (&MarkerUnits::parse ("", ()).map_err (|e| AttributeError::from (e))));
+        assert! (is_parse_error (&MarkerUnits::parse ("foo", ()).map_err (|e| AttributeError::from (e))));
     }
 
     #[test]
     fn parses_marker_units () {
-        assert_eq! (MarkerUnits::from_str ("userSpaceOnUse"), Ok (MarkerUnits::UserSpaceOnUse));
-        assert_eq! (MarkerUnits::from_str ("strokeWidth"),    Ok (MarkerUnits::StrokeWidth));
+        assert_eq! (MarkerUnits::parse ("userSpaceOnUse", ()), Ok (MarkerUnits::UserSpaceOnUse));
+        assert_eq! (MarkerUnits::parse ("strokeWidth", ()),    Ok (MarkerUnits::StrokeWidth));
     }
 
     #[test]
     fn parsing_invalid_marker_orient_yields_error () {
-        assert! (is_parse_error (&MarkerOrient::from_str ("").map_err (|e| AttributeError::from (e))));
-        assert! (is_parse_error (&MarkerOrient::from_str ("blah").map_err (|e| AttributeError::from (e))));
-        assert! (is_parse_error (&MarkerOrient::from_str ("45blah").map_err (|e| AttributeError::from (e))));
+        assert! (is_parse_error (&MarkerOrient::parse ("", ()).map_err (|e| AttributeError::from (e))));
+        assert! (is_parse_error (&MarkerOrient::parse ("blah", ()).map_err (|e| AttributeError::from (e))));
+        assert! (is_parse_error (&MarkerOrient::parse ("45blah", ()).map_err (|e| AttributeError::from (e))));
     }
 
     #[test]
     fn parses_marker_orient () {
-        assert_eq! (MarkerOrient::from_str ("auto"), Ok (MarkerOrient::Auto));
+        assert_eq! (MarkerOrient::parse ("auto", ()), Ok (MarkerOrient::Auto));
 
-        assert_eq! (MarkerOrient::from_str ("0"), Ok (MarkerOrient::Degrees (0.0)));
-        assert_eq! (MarkerOrient::from_str ("180"), Ok (MarkerOrient::Degrees (180.0)));
-        assert_eq! (MarkerOrient::from_str ("180deg"), Ok (MarkerOrient::Degrees (180.0)));
-        assert_eq! (MarkerOrient::from_str ("-400grad"), Ok (MarkerOrient::Degrees (-360.0)));
-        assert_eq! (MarkerOrient::from_str ("1rad"), Ok (MarkerOrient::Degrees (180.0 / PI)));
+        assert_eq! (MarkerOrient::parse ("0", ()), Ok (MarkerOrient::Degrees (0.0)));
+        assert_eq! (MarkerOrient::parse ("180", ()), Ok (MarkerOrient::Degrees (180.0)));
+        assert_eq! (MarkerOrient::parse ("180deg", ()), Ok (MarkerOrient::Degrees (180.0)));
+        assert_eq! (MarkerOrient::parse ("-400grad", ()), Ok (MarkerOrient::Degrees (-360.0)));
+        assert_eq! (MarkerOrient::parse ("1rad", ()), Ok (MarkerOrient::Degrees (180.0 / PI)));
     }
 }
 
@@ -795,7 +798,6 @@ mod parser_tests {
 mod directionality_tests {
     use std::f64::consts::*;
     use super::*;
-    extern crate cairo;
 
     fn test_bisection_angle (expected: f64,
                              incoming_vx: f64,
@@ -1147,7 +1149,6 @@ mod directionality_tests {
 #[cfg(test)]
 mod marker_tests {
     use super::*;
-    extern crate cairo;
 
     #[test]
     fn emits_for_open_subpath () {
