@@ -1,13 +1,8 @@
-extern crate libc;
-extern crate glib;
-
 use std::str;
 use std::str::Chars;
 use std::iter::Enumerate;
 use path_builder::*;
 
-extern crate cairo;
-
 struct PathParser<'b> {
     chars_enumerator: Enumerate<Chars<'b>>,
     lookahead: Option <char>, /* None if we are in EOF */
@@ -1008,8 +1003,6 @@ pub fn parse_path_into_builder (path_str: &str, builder: &mut RsvgPathBuilder) -
 mod tests {
     use super::*;
 
-    extern crate cairo;
-
     fn print_error (error: &ParseError, path_str: &str) {
         let prefix = "Error in \"";
 
