@@ -10,14 +10,15 @@ use cairo::MatrixTrait;
 use aspect_ratio::*;
 use drawing_ctx::RsvgDrawingCtx;
 use drawing_ctx;
-use error::*;
 use handle::RsvgHandle;
 use length::*;
 use node::*;
+use parsers::Parse;
 use property_bag;
 use property_bag::*;
 use util::*;
 use viewbox::*;
+use viewport::{ClipMode,draw_in_viewport};
 
 /***** NodeGroup *****/
 
@@ -133,39 +134,30 @@ impl NodeSvg {
     }
 }
 
-fn length_is_negative (length: &RsvgLength) -> bool {
-    // This is more or less a hack.  We don't care about a correct
-    // normalization; we just need to know if it would be negative.
-    // So, we pass bogus values just to be able to normalize.
-    length.hand_normalize (1.0, 1.0, 1.0) < 0.0
-}
-
 impl NodeTrait for NodeSvg {
     fn set_atts (&self, node: &RsvgNode, _: *const RsvgHandle, pbag: *const RsvgPropertyBag) -> NodeResult {
-        self.preserve_aspect_ratio.set (property_bag::parse_or_default (pbag, "preserveAspectRatio")?);
+        self.preserve_aspect_ratio.set (property_bag::parse_or_default (pbag, "preserveAspectRatio", (), None)?);
 
         // x & y attributes have no effect on outermost svg
         // http://www.w3.org/TR/SVG/struct.html#SVGElement
         if node.get_parent ().is_some () {
-            self.x.set (property_bag::length_or_default (pbag, "x", LengthDir::Horizontal)?);
-            self.y.set (property_bag::length_or_default (pbag, "y", LengthDir::Vertical)?);
+            self.x.set (property_bag::parse_or_default (pbag, "x", LengthDir::Horizontal, None)?);
+            self.y.set (property_bag::parse_or_default (pbag, "y", LengthDir::Vertical, None)?);
         }
 
-        let w = property_bag::length_or_value (pbag, "width", LengthDir::Horizontal, "100%")?;
-        if length_is_negative (&w) {
-            return Err (NodeError::value_error ("width", "Must not be negative"));
-        } else {
-            self.w.set (w);
-        }
+        self.w.set (property_bag::parse_or_value (pbag,
+                                                  "width",
+                                                  LengthDir::Horizontal,
+                                                  RsvgLength::parse ("100%", LengthDir::Horizontal).unwrap (),
+                                                  Some(RsvgLength::check_nonnegative))?);
 
-        let h = property_bag::length_or_value (pbag, "height", LengthDir::Vertical, "100%")?;
-        if length_is_negative (&h) {
-            return Err (NodeError::value_error ("height", "Must not be negative"));
-        } else {
-            self.h.set (h);
-        }
+        self.h.set (property_bag::parse_or_value (pbag,
+                                                  "height",
+                                                  LengthDir::Vertical,
+                                                  RsvgLength::parse ("100%", LengthDir::Vertical).unwrap (),
+                                                  Some(RsvgLength::check_nonnegative))?);
 
-        self.vbox.set (property_bag::parse_or_none (pbag, "viewBox")?);
+        self.vbox.set (property_bag::parse_or_none (pbag, "viewBox", (), None)?);
 
         // The "style" sub-element is not loaded yet here, so we need
         // to store other attributes to be applied later.
@@ -174,66 +166,27 @@ impl NodeTrait for NodeSvg {
         Ok (())
     }
 
-    fn draw (&self, node: &RsvgNode, draw_ctx: *const RsvgDrawingCtx, dominate: i32) {
+    fn draw (&self, node: &RsvgNode, draw_ctx: *const RsvgDrawingCtx, _dominate: i32) {
         let nx = self.x.get ().normalize (draw_ctx);
         let ny = self.y.get ().normalize (draw_ctx);
         let nw = self.w.get ().normalize (draw_ctx);
         let nh = self.h.get ().normalize (draw_ctx);
 
-        // width or height set to 0 disables rendering of the element
-        // https://www.w3.org/TR/SVG/struct.html#SVGElementWidthAttribute
-        if double_equals (nw, 0.0) || double_equals (nh, 0.0) {
-            return;
-        }
-
-        drawing_ctx::state_reinherit_top (draw_ctx, node.get_state (), dominate);
-
         let state = drawing_ctx::get_current_state (draw_ctx);
+        let do_clip = !drawing_ctx::state_is_overflow (state) && node.get_parent ().is_some ();
 
-        let affine_old = drawing_ctx::get_current_state_affine (draw_ctx);
-
-        if let Some (vbox) = self.vbox.get () {
-            // viewBox width==0 or height==0 disables rendering of the element
-            // https://www.w3.org/TR/SVG/coords.html#ViewBoxAttribute
-            if double_equals (vbox.0.width, 0.0) || double_equals (vbox.0.height, 0.0) {
-                return;
-            }
-
-            let (x, y, w, h) = self.preserve_aspect_ratio.get ().compute (vbox.0.width, vbox.0.height,
-                                                                          nx, ny, nw, nh);
-
-            let mut affine = affine_old;
-            affine.translate (x, y);
-            affine.scale (w / vbox.0.width, h / vbox.0.height);
-            affine.translate (-vbox.0.x, -vbox.0.y);
-            drawing_ctx::set_current_state_affine (draw_ctx, affine);
-
-            drawing_ctx::push_view_box (draw_ctx, vbox.0.width, vbox.0.height);
-        } else {
-            let mut affine = affine_old;
-            affine.translate (nx, ny);
-
-            drawing_ctx::set_current_state_affine (draw_ctx, affine);
-            drawing_ctx::push_view_box (draw_ctx, nw, nh);
-        }
-
-        let affine_new = drawing_ctx::get_current_state_affine (draw_ctx);
-
-        drawing_ctx::push_discrete_layer (draw_ctx);
-
-        // Bounding box addition must be AFTER the discrete layer
-        // push, which must be AFTER the transformation happens.
-
-        if !drawing_ctx::state_is_overflow (state) && node.get_parent ().is_some () {
-            drawing_ctx::set_current_state_affine (draw_ctx, affine_old);
-            drawing_ctx::add_clipping_rect (draw_ctx, nx, ny, nw, nh);
-            drawing_ctx::set_current_state_affine (draw_ctx, affine_new);
-        }
-
-        node.draw_children (draw_ctx, -1); // dominate==-1 so it won't reinherit or push a layer
-
-        drawing_ctx::pop_discrete_layer (draw_ctx);
-        drawing_ctx::pop_view_box (draw_ctx);
+        draw_in_viewport(nx, ny, nw, nh,
+                         ClipMode::ClipToViewport,
+                         do_clip,
+                         self.vbox.get(),
+                         self.preserve_aspect_ratio.get(),
+                         drawing_ctx::get_current_state_affine(draw_ctx),
+                         draw_ctx,
+                         || {
+                             drawing_ctx::state_push(draw_ctx);
+                             node.draw_children(draw_ctx, -1); // dominate==-1 so it won't reinherit or push a layer
+                             drawing_ctx::state_pop(draw_ctx);
+                         });
     }
 
     fn get_c_impl (&self) -> *const RsvgCNodeImpl {
@@ -277,38 +230,14 @@ impl NodeTrait for NodeUse {
     fn set_atts (&self, _: &RsvgNode, _: *const RsvgHandle, pbag: *const RsvgPropertyBag) -> NodeResult {
         *self.link.borrow_mut () = property_bag::lookup (pbag, "xlink:href");
 
-        self.x.set (property_bag::length_or_default (pbag, "x", LengthDir::Horizontal)?);
-        self.y.set (property_bag::length_or_default (pbag, "y", LengthDir::Vertical)?);
+        self.x.set (property_bag::parse_or_default (pbag, "x", LengthDir::Horizontal, None)?);
+        self.y.set (property_bag::parse_or_default (pbag, "y", LengthDir::Vertical, None)?);
 
-        let opt_w = property_bag::length_or_none (pbag, "width", LengthDir::Horizontal)?;
-        match opt_w {
-            Some (w) => {
-                if length_is_negative (&w) {
-                    return Err (NodeError::value_error ("width", "Must not be negative"));
-                } else {
-                    self.w.set (Some (w));
-                }
-            },
+        self.w.set (property_bag::parse_or_none (pbag, "width", LengthDir::Horizontal,
+                                                 Some(RsvgLength::check_nonnegative))?);
 
-            None => {
-                self.w.set (None);
-            }
-        }
-
-        let opt_h = property_bag::length_or_none (pbag, "height", LengthDir::Vertical)?;
-        match opt_h {
-            Some (h) => {
-                if length_is_negative (&h) {
-                    return Err (NodeError::value_error ("height", "Must not be negative"));
-                } else {
-                    self.h.set (Some (h));
-                }
-            },
-
-            None => {
-                self.h.set (None);
-            }
-        }
+        self.h.set (property_bag::parse_or_none (pbag, "height", LengthDir::Vertical,
+                                                 Some(RsvgLength::check_nonnegative))?);
 
         Ok (())
     }
@@ -347,16 +276,11 @@ impl NodeTrait for NodeUse {
 
         // width or height set to 0 disables rendering of the element
         // https://www.w3.org/TR/SVG/struct.html#UseElementWidthAttribute
-        if double_equals (nw, 0.0) {
+        if double_equals (nw, 0.0) || double_equals (nh, 0.0) {
             drawing_ctx::release_node (draw_ctx, raw_child);
             return;
         }
         
-        if double_equals (nh, 0.0) {
-            drawing_ctx::release_node (draw_ctx, raw_child);
-            return;
-        }
-
         drawing_ctx::state_reinherit_top (draw_ctx, node.get_state (), dominate);
 
         let state = drawing_ctx::get_current_state (draw_ctx);
@@ -366,52 +290,36 @@ impl NodeTrait for NodeUse {
             affine.translate (nx, ny);
             drawing_ctx::set_current_state_affine (draw_ctx, affine);
 
-            let boxed_child = box_node (child.clone ());
-
             drawing_ctx::push_discrete_layer (draw_ctx);
+
+            let boxed_child = box_node (child.clone ());
             drawing_ctx::draw_node_from_stack (draw_ctx, boxed_child, 1);
+            rsvg_node_unref (boxed_child);
+
+            drawing_ctx::release_node (draw_ctx, raw_child);
             drawing_ctx::pop_discrete_layer (draw_ctx);
         } else {
             child.with_impl (|symbol: &NodeSymbol| {
-                if let Some (vbox) = symbol.vbox.get () {
-                    let (x, y, w, h) = symbol.preserve_aspect_ratio.get ().compute (vbox.0.width, vbox.0.height,
-                                                                                    nx, ny, nw, nh);
+                let do_clip = !drawing_ctx::state_is_overflow (state)
+                    || (!drawing_ctx::state_has_overflow (state)
+                        && drawing_ctx::state_is_overflow (child.get_state ()));
 
-                    let mut affine = drawing_ctx::get_current_state_affine (draw_ctx);
-                    affine.translate (x, y);
-                    affine.scale (w / vbox.0.width, h / vbox.0.height);
-                    affine.translate (-vbox.0.x, -vbox.0.y);
-                    drawing_ctx::set_current_state_affine (draw_ctx, affine);
-
-                    drawing_ctx::push_view_box (draw_ctx, vbox.0.width, vbox.0.height);
-
-                    drawing_ctx::push_discrete_layer (draw_ctx);
-
-                    if !drawing_ctx::state_is_overflow (state) || (!drawing_ctx::state_has_overflow (state)
-                                                                   && drawing_ctx::state_is_overflow (child.get_state ())) {
-                        drawing_ctx::add_clipping_rect (draw_ctx, vbox.0.x, vbox.0.y, vbox.0.width, vbox.0.height);
-                    }
-                } else {
-                    let mut affine = drawing_ctx::get_current_state_affine (draw_ctx);
-                    affine.translate (nx, ny);
-                    drawing_ctx::set_current_state_affine (draw_ctx, affine);
-
-                    drawing_ctx::push_discrete_layer (draw_ctx);
-                    drawing_ctx::push_view_box (draw_ctx, nw, nh);
-                }
-
-                drawing_ctx::state_push (draw_ctx);
-
-                child.draw_children (draw_ctx, 1);
-
-                drawing_ctx::state_pop (draw_ctx);
-                drawing_ctx::pop_discrete_layer (draw_ctx);
-
-                drawing_ctx::pop_view_box (draw_ctx);
+                draw_in_viewport(nx, ny, nw, nh,
+                                 ClipMode::ClipToVbox,
+                                 do_clip,
+                                 symbol.vbox.get(),
+                                 symbol.preserve_aspect_ratio.get(),
+                                 drawing_ctx::get_current_state_affine(draw_ctx),
+                                 draw_ctx,
+                                 || {
+                                     drawing_ctx::state_push(draw_ctx);
+                                     child.draw_children(draw_ctx, 1);
+                                     drawing_ctx::state_pop(draw_ctx);
+                                 });
             });
-        }
 
-        drawing_ctx::release_node (draw_ctx, raw_child);
+            drawing_ctx::release_node (draw_ctx, raw_child);
+        }
     }
 
     fn get_c_impl (&self) -> *const RsvgCNodeImpl {
@@ -437,8 +345,8 @@ impl NodeSymbol {
 
 impl NodeTrait for NodeSymbol {
     fn set_atts (&self, _: &RsvgNode, _: *const RsvgHandle, pbag: *const RsvgPropertyBag) -> NodeResult {
-        self.preserve_aspect_ratio.set (property_bag::parse_or_default (pbag, "preserveAspectRatio")?);
-        self.vbox.set (property_bag::parse_or_none (pbag, "viewBox")?);
+        self.preserve_aspect_ratio.set (property_bag::parse_or_default (pbag, "preserveAspectRatio", (), None)?);
+        self.vbox.set (property_bag::parse_or_none (pbag, "viewBox", (), None)?);
 
         Ok (())
     }
